@@ -42,7 +42,7 @@
 // Old Todo:
 //  - Crypt/decrypt button in the userinterface
 //  - Setting if crypto should be on by default
-//  - Add a setting "Don´t encrypt files larger than xx because of performance 
+//  - Add a setting "Don´t encrypt files larger than xx because of performance
 //    reasons"
 
 namespace OCA\Encryption;
@@ -68,7 +68,7 @@ class Util {
 
 	//// DONE: new data filled files added via webdav get encrypted
 	//// DONE: new data filled files added via webdav are readable via webdav
-	//// DONE: reading unencrypted files when encryption is enabled works via 
+	//// DONE: reading unencrypted files when encryption is enabled works via
 	////       webdav
 	//// DONE: files created & encrypted via web ui are readable via webdav
 
@@ -96,10 +96,13 @@ class Util {
 	//// DONE: test new encryption with sharing
 	//// TODO: test new encryption with proxies
 
+	const MIGRATION_COMPLETED = 1;    // migration to new encryption completed
+	const MIGRATION_IN_PROGRESS = -1; // migration is running
+	const MIGRATION_OPEN = 0;         // user still needs to be migrated
+
 
 	private $view; // OC_FilesystemView object for filesystem operations
 	private $userId; // ID of the currently logged-in user
-	private $pwd; // User Password
 	private $client; // Client side encryption mode flag
 	private $publicKeyDir; // Dir containing all public user keys
 	private $encryptionDir; // Dir containing user's files_encryption
@@ -225,18 +228,21 @@ class Util {
 			// Generate keypair
 			$keypair = Crypt::createKeypair();
 
-			\OC_FileProxy::$enabled = false;
+			if ($keypair) {
 
-			// Save public key
-			$this->view->file_put_contents($this->publicKeyPath, $keypair['publicKey']);
+				\OC_FileProxy::$enabled = false;
 
-			// Encrypt private key with user pwd as passphrase
-			$encryptedPrivateKey = Crypt::symmetricEncryptFileContent($keypair['privateKey'], $passphrase);
+				// Encrypt private key with user pwd as passphrase
+				$encryptedPrivateKey = Crypt::symmetricEncryptFileContent($keypair['privateKey'], $passphrase);
 
-			// Save private key
-			$this->view->file_put_contents($this->privateKeyPath, $encryptedPrivateKey);
+				// Save key-pair
+				if ($encryptedPrivateKey) {
+					$this->view->file_put_contents($this->privateKeyPath, $encryptedPrivateKey);
+					$this->view->file_put_contents($this->publicKeyPath, $keypair['publicKey']);
+				}
 
-			\OC_FileProxy::$enabled = true;
+				\OC_FileProxy::$enabled = true;
+			}
 
 		} else {
 			// check if public-key exists but private-key is missing
@@ -289,7 +295,7 @@ class Util {
 	 */
 	public function recoveryEnabledForUser() {
 
-		$sql = 'SELECT `recovery_enabled` FROM `*PREFIX*encryption` WHERE  uid = ?';
+		$sql = 'SELECT `recovery_enabled` FROM `*PREFIX*encryption` WHERE `uid` = ?';
 
 		$args = array($this->userId);
 
@@ -302,7 +308,7 @@ class Util {
 		if (\OCP\DB::isError($result)) {
 			\OCP\Util::writeLog('Encryption library', \OC_DB::getErrorMessage($result), \OCP\Util::ERROR);
 		} else {
-			if($result->numRows() > 0) {
+			if ($result->numRows() > 0) {
 				$row = $result->fetchRow();
 				if (isset($row['recovery_enabled'])) {
 					$recoveryEnabled[] = $row['recovery_enabled'];
@@ -347,7 +353,7 @@ class Util {
 			// Create a new record instead
 		} else {
 
-			$sql = 'UPDATE `*PREFIX*encryption` SET recovery_enabled = ? WHERE uid = ?';
+			$sql = 'UPDATE `*PREFIX*encryption` SET `recovery_enabled` = ? WHERE `uid` = ?';
 
 			$args = array(
 				$enabled,
@@ -358,16 +364,20 @@ class Util {
 
 		$query = \OCP\DB::prepare($sql);
 
-		if ($query->execute($args)) {
-
-			return true;
-
-		} else {
-
+		if (\OCP\DB::isError($query)) {
+			\OCP\Util::writeLog('Encryption library', \OC_DB::getErrorMessage($query), \OCP\Util::ERROR);
 			return false;
-
 		}
 
+		$result = $query->execute($args);
+
+		if (\OCP\DB::isError($result)) {
+			\OCP\Util::writeLog('Encryption library', \OC_DB::getErrorMessage($result), \OCP\Util::ERROR);
+			return false;
+		}
+
+		return is_numeric($result);
+
 	}
 
 	/**
@@ -407,7 +417,7 @@ class Util {
 					$filePath = $directory . '/' . $this->view->getRelativePath('/' . $file);
 					$relPath = \OCA\Encryption\Helper::stripUserFilesPath($filePath);
 
-					// If the path is a directory, search 
+					// If the path is a directory, search
 					// its contents
 					if ($this->view->is_dir($filePath)) {
 
@@ -421,18 +431,17 @@ class Util {
 						// where they got re-enabled :/
 						\OC_FileProxy::$enabled = false;
 
-						$data = $this->view->file_get_contents($filePath);
-
+						$isEncryptedPath = $this->isEncryptedPath($filePath);
 						// If the file is encrypted
-						// NOTE: If the userId is 
-						// empty or not set, file will 
+						// NOTE: If the userId is
+						// empty or not set, file will
 						// detected as plain
 						// NOTE: This is inefficient;
 						// scanning every file like this
 						// will eat server resources :(
 						if (
 							Keymanager::getFileKey($this->view, $this->userId, $relPath)
-							&& Crypt::isCatfileContent($data)
+							&& $isEncryptedPath
 						) {
 
 							$found['encrypted'][] = array(
@@ -442,7 +451,7 @@ class Util {
 
 							// If the file uses old
 							// encryption system
-						} elseif ( Crypt::isLegacyEncryptedContent( $data, $relPath ) ) {
+						} elseif (Crypt::isLegacyEncryptedContent($isEncryptedPath, $relPath)) {
 
 							$found['legacy'][] = array(
 								'name' => $file,
@@ -573,14 +582,17 @@ class Util {
 		// get relative path
 		$relativePath = \OCA\Encryption\Helper::stripUserFilesPath($path);
 
-		if (isset($pathParts[2]) && $pathParts[2] === 'files' && $this->view->file_exists($path) && $this->isEncryptedPath($path)) {
+		if (isset($pathParts[2]) && $pathParts[2] === 'files' && $this->view->file_exists($path)
+			&& $this->isEncryptedPath($path)
+		) {
 
 			// get the size from filesystem
 			$fullPath = $this->view->getLocalFile($path);
-			$size = filesize($fullPath);
+			$size = $this->view->filesize($path);
 
 			// calculate last chunk nr
 			$lastChunkNr = floor($size / 8192);
+			$lastChunkSize = $size - ($lastChunkNr * 8192);
 
 			// open stream
 			$stream = fopen('crypt://' . $relativePath, "r");
@@ -593,7 +605,7 @@ class Util {
 				fseek($stream, $lastChunckPos);
 
 				// get the content of the last chunk
-				$lastChunkContent = fread($stream, 8192);
+				$lastChunkContent = fread($stream, $lastChunkSize);
 
 				// calc the real file size with the size of the last chunk
 				$realSize = (($lastChunkNr * 6126) + strlen($lastChunkContent));
@@ -643,7 +655,7 @@ class Util {
 		return $result;
 	}
 
-	
+
 	/**
 	 * @param $path
 	 * @return bool
@@ -686,29 +698,37 @@ class Util {
 				//relative to data/<user>/file
 				$relPath = $plainFile['path'];
 
+				//get file info
+				$fileInfo = \OC\Files\Filesystem::getFileInfo($plainFile['path']);
+
 				//relative to /data
-				$rawPath = '/'.$this->userId . '/files/' . $plainFile['path'];
+				$rawPath = '/' . $this->userId . '/files/' . $plainFile['path'];
 
 				// Open plain file handle for binary reading
-				$plainHandle = $this->view->fopen( $rawPath, 'rb' );
+				$plainHandle = $this->view->fopen($rawPath, 'rb');
 
 				// Open enc file handle for binary writing, with same filename as original plain file
-				$encHandle = fopen( 'crypt://' . $relPath.'.tmp', 'wb' );
+				$encHandle = fopen('crypt://' . $relPath . '.part', 'wb');
 
 				// Move plain file to a temporary location
-				$size = stream_copy_to_stream( $plainHandle, $encHandle );
+				$size = stream_copy_to_stream($plainHandle, $encHandle);
 
 				fclose($encHandle);
 
 				$fakeRoot = $this->view->getRoot();
-				$this->view->chroot('/'.$this->userId.'/files');
+				$this->view->chroot('/' . $this->userId . '/files');
 
-				$this->view->rename($relPath . '.tmp', $relPath);
+				$this->view->rename($relPath . '.part', $relPath);
 
 				$this->view->chroot($fakeRoot);
 
 				// Add the file to the cache
-				\OC\Files\Filesystem::putFileInfo( $relPath, array( 'encrypted' => true, 'size' => $size, 'unencrypted_size' => $size ) );
+				\OC\Files\Filesystem::putFileInfo($relPath, array(
+					'encrypted' => true,
+					'size' => $size,
+					'unencrypted_size' => $size,
+					'etag' => $fileInfo['etag']
+				));
 			}
 
 			// Encrypt legacy encrypted files
@@ -722,40 +742,28 @@ class Util {
 					// Fetch data from file
 					$legacyData = $this->view->file_get_contents($legacyFile['path']);
 
-					$sharingEnabled = \OCP\Share::isEnabled();
-
-					// if file exists try to get sharing users
-					if ($this->view->file_exists($legacyFile['path'])) {
-						$uniqueUserIds = $this->getSharingUsersArray($sharingEnabled, $legacyFile['path'], $this->userId);
-					} else {
-						$uniqueUserIds[] = $this->userId;
-					}
-
-					// Fetch public keys for all users who will share the file
-					$publicKeys = Keymanager::getPublicKeys($this->view, $uniqueUserIds);
-
-					// Recrypt data, generate catfile
-					$recrypted = Crypt::legacyKeyRecryptKeyfile( $legacyData, $legacyPassphrase, $publicKeys );
+					// decrypt data, generate catfile
+					$decrypted = Crypt::legacyBlockDecrypt($legacyData, $legacyPassphrase);
 
 					$rawPath = $legacyFile['path'];
-					$relPath = \OCA\Encryption\Helper::stripUserFilesPath($rawPath);
 
-					// Save keyfile
-					Keymanager::setFileKey($this->view, $relPath, $this->userId, $recrypted['filekey']);
+					// enable proxy the ensure encryption is handled
+					\OC_FileProxy::$enabled = true;
 
-					// Save sharekeys to user folders
-					Keymanager::setShareKeys($this->view, $relPath, $recrypted['sharekeys']);
+					// Open enc file handle for binary writing, with same filename as original plain file
+					$encHandle = $this->view->fopen( $rawPath, 'wb' );
 
-					// Overwrite the existing file with the encrypted one
-					$this->view->file_put_contents($rawPath, $recrypted['data']);
+					if (is_resource($encHandle)) {
 
-					$size = strlen($recrypted['data']);
+						// write data to stream
+						fwrite($encHandle, $decrypted);
 
-					// Add the file to the cache
-					\OC\Files\Filesystem::putFileInfo($rawPath, array(
-																	 'encrypted' => true,
-																	 'size' => $size
-																), '');
+						// close stream
+						fclose($encHandle);
+					}
+
+					// disable proxy to prevent file being encrypted twice
+					\OC_FileProxy::$enabled = false;
 				}
 			}
 
@@ -831,7 +839,7 @@ class Util {
 		if (\OCP\DB::isError($result)) {
 			\OCP\Util::writeLog('Encryption library', \OC_DB::getErrorMessage($result), \OCP\Util::ERROR);
 		} else {
-			if($result->numRows() > 0) {
+			if ($result->numRows() > 0) {
 				$row = $result->fetchRow();
 				$path = substr($row['path'], strlen('files'));
 			}
@@ -913,7 +921,7 @@ class Util {
 			&& !Keymanager::getShareKey($this->view, $this->userId, $filePath) // NOTE: we can't use isShared() here because it's a post share hook so it always returns true
 		) {
 
-			// The file has no shareKey, and its keyfile must be 
+			// The file has no shareKey, and its keyfile must be
 			// decrypted conventionally
 			$plainKeyfile = Crypt::keyDecrypt($encKeyfile, $privateKey);
 
@@ -1003,18 +1011,19 @@ class Util {
 			\OC_Appconfig::getValue('files_encryption', 'recoveryAdminEnabled')
 			&& $this->recoveryEnabledForUser()
 		) {
-
 			$recoveryEnabled = true;
-
 		} else {
-
 			$recoveryEnabled = false;
-
 		}
 
 		// Make sure that a share key is generated for the owner too
 		list($owner, $ownerPath) = $this->getUidAndFilename($filePath);
 
+		$pathinfo = pathinfo($ownerPath);
+		if(array_key_exists('extension', $pathinfo) && $pathinfo['extension'] === 'part') {
+			$ownerPath = $pathinfo['dirname'] . '/' . $pathinfo['filename'];
+		}
+
 		$userIds = array();
 		if ($sharingEnabled) {
 
@@ -1027,23 +1036,28 @@ class Util {
 
 		}
 
-		// If recovery is enabled, add the 
+		// If recovery is enabled, add the
 		// Admin UID to list of users to share to
 		if ($recoveryEnabled) {
-
 			// Find recoveryAdmin user ID
 			$recoveryKeyId = \OC_Appconfig::getValue('files_encryption', 'recoveryKeyId');
-
 			// Add recoveryAdmin to list of users sharing
 			$userIds[] = $recoveryKeyId;
-
 		}
 
 		// add current user if given
 		if ($currentUserId !== false) {
-
 			$userIds[] = $currentUserId;
+		}
 
+		// check if it is a group mount
+		if (\OCP\App::isEnabled("files_external")) {
+			$mount = \OC_Mount_Config::getSystemMountPoints();
+			foreach ($mount as $mountPoint => $data) {
+				if ($mountPoint == substr($ownerPath, 1, strlen($mountPoint))) {
+					$userIds = array_merge($userIds, $this->getUserWithAccessToMountPoint($data['applicable']['users'], $data['applicable']['groups']));
+				}
+			}
 		}
 
 		// Remove duplicate UIDs
@@ -1053,43 +1067,75 @@ class Util {
 
 	}
 
-	/**
-	 * @brief Set file migration status for user
-	 * @param $status
-	 * @return bool
-	 */
-	public function setMigrationStatus($status) {
-
-		$sql = 'UPDATE `*PREFIX*encryption` SET migration_status = ? WHERE uid = ?';
-
-		$args = array(
-			$status,
-			$this->userId
-		);
-
-		$query = \OCP\DB::prepare($sql);
-
-		if ($query->execute($args)) {
-
-			return true;
-
+	private function getUserWithAccessToMountPoint($users, $groups) {
+		$result = array();
+		if (in_array('all', $users)) {
+			$result = \OCP\User::getUsers();
 		} else {
-
-			return false;
-
+			$result = array_merge($result, $users);
+			foreach ($groups as $group) {
+				$result = array_merge($result, \OC_Group::usersInGroup($group));
+			}
 		}
 
+		return $result;
 	}
 
 	/**
-	 * @brief Check whether pwd recovery is enabled for a given user
-	 * @return bool 1 = yes, 0 = no, false = no record
+	 * @brief start migration mode to initially encrypt users data
+	 * @return boolean
+	 */
+	public function beginMigration() {
+
+		$return = false;
+
+		$sql = 'UPDATE `*PREFIX*encryption` SET `migration_status` = ? WHERE `uid` = ? and `migration_status` = ?';
+		$args = array(self::MIGRATION_IN_PROGRESS, $this->userId, self::MIGRATION_OPEN);
+		$query = \OCP\DB::prepare($sql);
+		$manipulatedRows = $query->execute($args);
+
+		if ($manipulatedRows === 1) {
+			$return = true;
+			\OCP\Util::writeLog('Encryption library', "Start migration to encryption mode for " . $this->userId, \OCP\Util::INFO);
+		} else {
+			\OCP\Util::writeLog('Encryption library', "Could not activate migration mode for " . $this->userId . ". Probably another process already started the initial encryption", \OCP\Util::WARN);
+		}
+
+		return $return;
+	}
+
+	/**
+	 * @brief close migration mode after users data has been encrypted successfully
+	 * @return boolean
+	 */
+	public function finishMigration() {
+
+		$return = false;
+
+		$sql = 'UPDATE `*PREFIX*encryption` SET `migration_status` = ? WHERE `uid` = ? and `migration_status` = ?';
+		$args = array(self::MIGRATION_COMPLETED, $this->userId, self::MIGRATION_IN_PROGRESS);
+		$query = \OCP\DB::prepare($sql);
+		$manipulatedRows = $query->execute($args);
+
+		if ($manipulatedRows === 1) {
+			$return = true;
+			\OCP\Util::writeLog('Encryption library', "Finish migration successfully for " . $this->userId, \OCP\Util::INFO);
+		} else {
+			\OCP\Util::writeLog('Encryption library', "Could not deactivate migration mode for " . $this->userId, \OCP\Util::WARN);
+		}
+
+		return $return;
+	}
+
+	/**
+	 * @brief check if files are already migrated to the encryption system
+	 * @return migration status, false = in case of no record
 	 * @note If records are not being returned, check for a hidden space
 	 *       at the start of the uid in db
 	 */
 	public function getMigrationStatus() {
 
-		$sql = 'SELECT `migration_status` FROM `*PREFIX*encryption`  WHERE uid = ?';
+		$sql = 'SELECT `migration_status` FROM `*PREFIX*encryption` WHERE `uid` = ?';
 
 		$args = array($this->userId);
 
@@ -1102,7 +1148,7 @@ class Util {
 		if (\OCP\DB::isError($result)) {
 			\OCP\Util::writeLog('Encryption library', \OC_DB::getErrorMessage($result), \OCP\Util::ERROR);
 		} else {
-			if($result->numRows() > 0) {
+			if ($result->numRows() > 0) {
 				$row = $result->fetchRow();
 				if (isset($row['migration_status'])) {
 					$migrationStatus[] = $row['migration_status'];
@@ -1112,14 +1158,11 @@ class Util {
 
 		// If no record is found
 		if (empty($migrationStatus)) {
-
+			\OCP\Util::writeLog('Encryption library', "Could not get migration status for " . $this->userId . ", no record found", \OCP\Util::ERROR);
 			return false;
-
 			// If a record is found
 		} else {
-
-			return $migrationStatus[0];
-
+			return (int)$migrationStatus[0];
 		}
 
 	}
@@ -1134,8 +1177,25 @@ class Util {
 	 */
 	public function getUidAndFilename($path) {
 
+		$pathinfo = pathinfo($path);
+		$partfile = false;
+		$parentFolder = false;
+		if (array_key_exists('extension', $pathinfo) && $pathinfo['extension'] === 'part') {
+			// if the real file exists we check this file
+			$filePath = $this->userFilesDir . '/' .$pathinfo['dirname'] . '/' . $pathinfo['filename'];
+			if ($this->view->file_exists($filePath)) {
+				$pathToCheck = $pathinfo['dirname'] . '/' . $pathinfo['filename'];
+			} else { // otherwise we look for the parent
+				$pathToCheck = $pathinfo['dirname'];
+				$parentFolder = true;
+			}
+			$partfile = true;
+		} else {
+			$pathToCheck = $path;
+		}
+
 		$view = new \OC\Files\View($this->userFilesDir);
-		$fileOwnerUid = $view->getOwner($path);
+		$fileOwnerUid = $view->getOwner($pathToCheck);
 
 		// handle public access
 		if ($this->isPublic) {
@@ -1164,24 +1224,29 @@ class Util {
 				$filename = $path;
 
 			} else {
-
-				$info = $view->getFileInfo($path);
+				$info = $view->getFileInfo($pathToCheck);
 				$ownerView = new \OC\Files\View('/' . $fileOwnerUid . '/files');
 
 				// Fetch real file path from DB
-				$filename = $ownerView->getPath($info['fileid']); // TODO: Check that this returns a path without including the user data dir
+				$filename = $ownerView->getPath($info['fileid']);
+				if ($parentFolder) {
+					$filename = $filename . '/'. $pathinfo['filename'];
+				}
+
+				if ($partfile) {
+					$filename = $filename . '.' . $pathinfo['extension'];
+				}
 
 			}
 
 			return array(
 				$fileOwnerUid,
-				$filename
+				\OC_Filesystem::normalizePath($filename)
 			);
 		}
-
-
 	}
 
+
 	/**
 	 * @brief go recursively through a dir and collect all files and sub files.
 	 * @param string $dir relative to the users files folder
@@ -1191,7 +1256,8 @@ class Util {
 
 		$result = array();
 
-		$content = $this->view->getDirectoryContent(\OC\Files\Filesystem::normalizePath($this->userFilesDir . '/' . $dir));
+		$content = $this->view->getDirectoryContent(\OC\Files\Filesystem::normalizePath(
+			$this->userFilesDir . '/' . $dir));
 
 		// handling for re shared folders
 		$pathSplit = explode('/', $dir);
@@ -1252,7 +1318,7 @@ class Util {
 		if (\OCP\DB::isError($result)) {
 			\OCP\Util::writeLog('Encryption library', \OC_DB::getErrorMessage($result), \OCP\Util::ERROR);
 		} else {
-			if($result->numRows() > 0) {
+			if ($result->numRows() > 0) {
 				$row = $result->fetchRow();
 			}
 		}
@@ -1278,7 +1344,7 @@ class Util {
 		if (\OCP\DB::isError($result)) {
 			\OCP\Util::writeLog('Encryption library', \OC_DB::getErrorMessage($result), \OCP\Util::ERROR);
 		} else {
-			if($result->numRows() > 0) {
+			if ($result->numRows() > 0) {
 				$row = $result->fetchRow();
 			}
 		}
@@ -1303,7 +1369,7 @@ class Util {
 		if (\OCP\DB::isError($result)) {
 			\OCP\Util::writeLog('Encryption library', \OC_DB::getErrorMessage($result), \OCP\Util::ERROR);
 		} else {
-			if($result->numRows() > 0) {
+			if ($result->numRows() > 0) {
 				$source = $result->fetchRow();
 			}
 		}
@@ -1324,7 +1390,7 @@ class Util {
 				if (\OCP\DB::isError($result)) {
 					\OCP\Util::writeLog('Encryption library', \OC_DB::getErrorMessage($result), \OCP\Util::ERROR);
 				} else {
-					if($result->numRows() > 0) {
+					if ($result->numRows() > 0) {
 						$item = $result->fetchRow();
 					}
 				}
@@ -1372,26 +1438,24 @@ class Util {
 	 */
 	public function checkRecoveryPassword($password) {
 
+		$result = false;
 		$pathKey = '/owncloud_private_key/' . $this->recoveryKeyId . ".private.key";
-		$pathControlData = '/control-file/controlfile.enc';
 
 		$proxyStatus = \OC_FileProxy::$enabled;
 		\OC_FileProxy::$enabled = false;
 
 		$recoveryKey = $this->view->file_get_contents($pathKey);
 
-		$decryptedRecoveryKey = Crypt::symmetricDecryptFileContent($recoveryKey, $password);
+		$decryptedRecoveryKey = Crypt::decryptPrivateKey($recoveryKey, $password);
 
-		$controlData = $this->view->file_get_contents($pathControlData);
-		$decryptedControlData = Crypt::keyDecrypt($controlData, $decryptedRecoveryKey);
+		if ($decryptedRecoveryKey) {
+			$result = true;
+		}
 
 		\OC_FileProxy::$enabled = $proxyStatus;
 
-		if ($decryptedControlData === 'ownCloud') {
-			return true;
-		}
 
-		return false;
+		return $result;
 	}
 
 	/**
@@ -1520,7 +1584,7 @@ class Util {
 
 		$encryptedKey = $this->view->file_get_contents(
 			'/owncloud_private_key/' . $this->recoveryKeyId . '.private.key');
-		$privateKey = Crypt::symmetricDecryptFileContent($encryptedKey, $recoveryPassword);
+		$privateKey = Crypt::decryptPrivateKey($encryptedKey, $recoveryPassword);
 
 		\OC_FileProxy::$enabled = $proxyStatus;
 
@@ -1536,7 +1600,7 @@ class Util {
 		list($storage, $internalPath) = \OC\Files\Cache\Cache::getById($id);
 		$mount = \OC\Files\Filesystem::getMountByStorageId($storage);
 		$mountPoint = $mount[0]->getMountPoint();
-		$path = \OC\Files\Filesystem::normalizePath($mountPoint.'/'.$internalPath);
+		$path = \OC\Files\Filesystem::normalizePath($mountPoint . '/' . $internalPath);
 
 		// reformat the path to be relative e.g. /user/files/folder becomes /folder/
 		$relativePath = \OCA\Encryption\Helper::stripUserFilesPath($path);
@@ -1544,4 +1608,21 @@ class Util {
 		return $relativePath;
 	}
 
+	/**
+	 * @brief check if the file is stored on a system wide mount point
+	 * @param $path relative to /data/user with leading '/'
+	 * @return boolean
+	 */
+	public function isSystemWideMountPoint($path) {
+		if (\OCP\App::isEnabled("files_external")) {
+			$mount = \OC_Mount_Config::getSystemMountPoints();
+			foreach ($mount as $mountPoint => $data) {
+				if ($mountPoint == substr($path, 1, strlen($mountPoint))) {
+					return true;
+				}
+			}
+		}
+		return false;
+	}
+
 }
