@@ -185,7 +185,7 @@ class OC_DB {
 					} else {
 							$dsn='sqlsrv:Server='.$host.';Database='.$name;
 					}
-					break;                    
+					break;
 				default:
 					return false;
 			}
@@ -273,18 +273,13 @@ class OC_DB {
 					break;
 				case 'oci':
 					$dsn = array(
-							'phptype'  => 'oci8',
-							'username' => $user,
-							'password' => $pass,
-							'charset' => 'AL32UTF8',
+						'phptype'  => 'oci8',
+						'username' => $user,
+						'password' => $pass,
+						'service'  => $name,
+						'hostspec' => $host,
+						'charset' => 'AL32UTF8',
 					);
-					if ($host != '') {
-						$dsn['hostspec'] = $host;
-						$dsn['database'] = $name;
-					} else { // use dbname for hostspec
-						$dsn['hostspec'] = $name;
-						$dsn['database'] = $user;
-					}
 					break;
 				case 'mssql':
 					$dsn = array(
@@ -330,11 +325,12 @@ class OC_DB {
 	 * @param string $query Query string
 	 * @param int $limit
 	 * @param int $offset
+	 * @param bool $isManipulation
 	 * @return MDB2_Statement_Common prepared SQL query
 	 *
 	 * SQL query via MDB2 prepare(), needs to be execute()'d!
 	 */
-	static public function prepare( $query , $limit=null, $offset=null ) {
+	static public function prepare( $query , $limit = null, $offset = null, $isManipulation = null) {
 
 		if (!is_null($limit) && $limit != -1) {
 			if (self::$backend == self::BACKEND_MDB2) {
@@ -369,12 +365,23 @@ class OC_DB {
 		$query = self::processQuery( $query );
 
 		self::connect();
+
+		if ($isManipulation === null) {
+			//try to guess, so we return the number of rows on manipulations
+			$isManipulation = self::isManipulation($query);
+		}
+
 		// return the result
 		if(self::$backend==self::BACKEND_MDB2) {
-			$result = self::$connection->prepare( $query );
+			// differentiate between query and manipulation
+			if ($isManipulation) {
+				$result = self::$connection->prepare( $query, null, MDB2_PREPARE_MANIP );
+			} else {
+				$result = self::$connection->prepare( $query, null, MDB2_PREPARE_RESULT );
+			}
 
 			// Die if we have an error (error means: bad query, not 0 results!)
-			if( PEAR::isError($result)) {
+			if( self::isError($result)) {
 				throw new DatabaseException($result->getMessage(), $query);
 			}
 		}else{
@@ -383,7 +390,8 @@ class OC_DB {
 			}catch(PDOException $e) {
 				throw new DatabaseException($e->getMessage(), $query);
 			}
-			$result=new PDOStatementWrapper($result);
+			// differentiate between query and manipulation
+			$result = new PDOStatementWrapper($result, $isManipulation);
 		}
 		if ((is_null($limit) || $limit == -1) and self::$cachingEnabled ) {
 			$type = OC_Config::getValue( "dbtype", "sqlite" );
@@ -394,6 +402,32 @@ class OC_DB {
 		return $result;
 	}
 
+	/**
+	 * tries to guess the type of statement based on the first 10 characters
+	 * the current check allows some whitespace but does not work with IF EXISTS or other more complex statements
+	 *
+	 * @param string $sql
+	 */
+	static public function isManipulation( $sql ) {
+		$selectOccurence = stripos ($sql, "SELECT");
+		if ($selectOccurence !== false && $selectOccurence < 10) {
+			return false;
+		}
+		$insertOccurence = stripos ($sql, "INSERT");
+		if ($insertOccurence !== false && $insertOccurence < 10) {
+			return true;
+		}
+		$updateOccurence = stripos ($sql, "UPDATE");
+		if ($updateOccurence !== false && $updateOccurence < 10) {
+			return true;
+		}
+		$deleteOccurance = stripos ($sql, "DELETE");
+		if ($deleteOccurance !== false && $deleteOccurance < 10) {
+			return true;
+		}
+		return false;
+	}
+
 	/**
 	 * @brief gets last value of autoincrement
 	 * @param string $table The optional table name (will replace *PREFIX*) and add sequence suffix
@@ -411,14 +445,13 @@ class OC_DB {
 			$query = self::prepare('SELECT lastval() AS id');
 			$row = $query->execute()->fetchRow();
 			return $row['id'];
-		}
-		if( $type == 'mssql' ) {
+		} else if( $type === 'mssql' || $type === 'oci') {
 			if($table !== null) {
 				$prefix = OC_Config::getValue( "dbtableprefix", "oc_" );
 				$table = str_replace( '*PREFIX*', $prefix, $table );
 			}
 			return self::$connection->lastInsertId($table);
-		}else{
+		} else {
 			if($table !== null) {
 				$prefix = OC_Config::getValue( "dbtableprefix", "oc_" );
 				$suffix = OC_Config::getValue( "dbsequencesuffix", "_id_seq" );
@@ -503,7 +536,7 @@ class OC_DB {
 		 * http://www.sqlite.org/lang_createtable.html
 		 * http://docs.oracle.com/cd/B19306_01/server.102/b14200/functions037.htm
 		 */
-		if( $CONFIG_DBTYPE == 'pgsql' ) { //mysql support it too but sqlite doesn't
+		if( $CONFIG_DBTYPE === 'pgsql' || $CONFIG_DBTYPE === 'oci' ) { //mysql support it too but sqlite doesn't
 			$content = str_replace( '<default>0000-00-00 00:00:00</default>',
 				'<default>CURRENT_TIMESTAMP</default>', $content );
 		}
@@ -551,10 +584,40 @@ class OC_DB {
 
 		self::connectScheme();
 
+		if(OC_Config::getValue('dbtype', 'sqlite')==='oci') {
+			//set dbname, it is unset because oci uses 'service' to connect
+			self::$schema->db->database_name=self::$schema->db->dsn['username'];
+
+			$installedVersion = \OC_Config::getValue('version', '0.0.0');
+			if (version_compare($installedVersion, '5.0.15', '<=')) {
+				//configvalue in oc_appconfig must be allowed to contain NULL
+				//because it is a CLOB, we need to move heaven and earth a bit
+				//i.e. rename it and copy values
+				$ociHandleAppconfig = true;
+
+				$query = \OCP\DB::prepare('SELECT * FROM `*PREFIX*appconfig`');
+				$result = $query->execute();
+				if(\OCP\DB::isError($result)) {
+					unset($ociHandleAppconfig);
+					throw new Exception('Cannot read from table appconfig:'.
+						$result->getMessage());
+				}
+				$ociAppconfigContent = $result->fetchAll();
+
+				$query = \OCP\DB::prepare('ALTER TABLE `*PREFIX*appconfig`
+					RENAME COLUMN `configvalue` TO `configvalue_tmp`');
+				$result = $query->execute();
+				if(\OCP\DB::isError($result)) {
+					OC_Log::write('core', 'Could not alter appconfig.'.
+						', DB upgrade may fail.', OC_Log::WARN);
+				}
+			}
+		}
+
 		// read file
 		$content = file_get_contents( $file );
 
-		$previousSchema = self::$schema->getDefinitionFromDatabase();
+		$previousSchema = self::$schema->getDefinitionFromDatabase($CONFIG_DBTABLEPREFIX);
 		if (PEAR::isError($previousSchema)) {
 			$error = $previousSchema->getMessage();
 			$detail = $previousSchema->getDebugInfo();
@@ -579,6 +642,12 @@ class OC_DB {
 			$content = str_replace( '<default>0000-00-00 00:00:00</default>',
 				'<default>CURRENT_TIMESTAMP</default>', $content );
 		}
+		if(OC_Config::getValue('dbtype', 'sqlite')==='oci') {
+			unset($previousSchema['charset']); //or MDB2 tries SHUTDOWN IMMEDIATE
+			$oldname = $previousSchema['name'];
+			$previousSchema['name']=OC_Config::getValue( "dbuser", $oldname );
+			//TODO check identifiers are at most 30 chars long
+		}
 		file_put_contents( $file2, $content );
 		$op = self::$schema->updateDatabase($file2, $previousSchema, array(), false);
 
@@ -592,6 +661,25 @@ class OC_DB {
 			OC_Log::write('core', $message, OC_Log::FATAL);
 			throw new Exception($message);
 		}
+
+		if(isset($ociHandleAppconfig) && isset($ociAppconfigContent)) {
+			$query = \OCP\DB::prepare('
+				UPDATE `*PREFIX*appconfig`
+				SET `configvalue` = ?
+				WHERE
+					`appid` = ?
+					AND `configkey` = ?
+			');
+			foreach($ociAppconfigContent as $row) {
+				$r = $query->execute(array(
+					$row['configvalue'], $row['appid'], $row['configkey']
+				));
+				if(\OCP\DB::isError($r)) {
+					throw new Exception($r->getMessage());
+				}
+			}
+		}
+
 		return true;
 	}
 
@@ -620,7 +708,7 @@ class OC_DB {
 	 * @brief Insert a row if a matching row doesn't exists.
 	 * @param string $table. The table to insert into in the form '*PREFIX*tableName'
 	 * @param array $input. An array of fieldname/value pairs
-	 * @returns The return value from PDOStatementWrapper->execute()
+	 * @returns int number of updated rows
 	 */
 	public static function insertIfNotExist($table, $input) {
 		self::connect();
@@ -646,6 +734,10 @@ class OC_DB {
 			try {
 				$stmt = self::prepare($query);
 				$result = $stmt->execute($inserts);
+				if (self::isError($result)) {
+					OC_Log::write('core', self::getErrorMessage($result), OC_Log::FATAL);
+					OC_Template::printErrorPage( $entry );
+				}
 
 			} catch(PDOException $e) {
 				$entry = 'DB Error: "'.$e->getMessage() . '"<br />';
@@ -660,7 +752,7 @@ class OC_DB {
 					. implode('`,`', array_keys($input)) . '`) VALUES('
 					. str_repeat('?,', count($input)-1).'? ' . ')';
 			} else {
-				return true;
+				return 0;
 			}
 		} elseif( $type == 'pgsql' || $type == 'oci' || $type == 'mysql' || $type == 'mssql') {
 			$query = 'INSERT INTO `' .$table . '` (`'
@@ -677,7 +769,18 @@ class OC_DB {
 		}
 
 		try {
-			$result = self::prepare($query);
+			$stmt = self::prepare($query);
+			if (self::isError($stmt)) {
+				$msg = self::getErrorMessage($stmt);
+				OC_Log::write('core', $msg, OC_Log::FATAL);
+				OC_Template::printErrorPage( $msg );
+			}
+			$result = $stmt->execute($inserts);
+			if (self::isError($result)) {
+				$msg = self::getErrorMessage($result);
+				OC_Log::write('core', $msg, OC_Log::FATAL);
+				OC_Template::printErrorPage( $msg );
+			}
 		} catch(PDOException $e) {
 			$entry = 'DB Error: "'.$e->getMessage() . '"<br />';
 			$entry .= 'Offending command was: ' . $query.'<br />';
@@ -686,7 +789,7 @@ class OC_DB {
 			OC_Template::printErrorPage( $entry );
 		}
 
-		return $result->execute($inserts);
+		return $result;
 	}
 
 	/**
@@ -721,15 +824,16 @@ class OC_DB {
 		}elseif( $type == 'oci'  ) {
 			$query = str_replace( '`', '"', $query );
 			$query = str_ireplace( 'NOW()', 'CURRENT_TIMESTAMP', $query );
+			$query = str_ireplace( 'UNIX_TIMESTAMP()', '((CAST(SYS_EXTRACT_UTC(systimestamp) AS DATE))-TO_DATE(\'1970101000000\',\'YYYYMMDDHH24MiSS\'))*24*3600', $query );
 		}elseif( $type == 'mssql' ) {
 			$query = preg_replace( "/\`(.*?)`/", "[$1]", $query );
-			$query = str_replace( 'NOW()', 'CURRENT_TIMESTAMP', $query );
-			$query = str_replace( 'now()', 'CURRENT_TIMESTAMP', $query );
+			$query = str_ireplace( 'NOW()', 'CURRENT_TIMESTAMP', $query );
 			$query = str_replace( 'LENGTH(', 'LEN(', $query );
 			$query = str_replace( 'SUBSTR(', 'SUBSTRING(', $query );
-            
-            $query = self::fixLimitClauseForMSSQL($query);
-        }
+			$query = str_ireplace( 'UNIX_TIMESTAMP()', 'DATEDIFF(second,{d \'1970-01-01\'},GETDATE())', $query );
+
+			$query = self::fixLimitClauseForMSSQL($query);
+		}
 
 		// replace table name prefix
 		$query = str_replace( '*PREFIX*', $prefix, $query );
@@ -739,11 +843,11 @@ class OC_DB {
 
     private static function fixLimitClauseForMSSQL($query) {
         $limitLocation = stripos ($query, "LIMIT");
-        
+
         if ( $limitLocation === false ) {
             return $query;
-        } 
-        
+        }
+
         // total == 0 means all results - not zero results
         //
         // First number is either total or offset, locate it by first space
@@ -790,7 +894,7 @@ class OC_DB {
         }
         return $query;
     }
-    
+
 	/**
 	 * @brief drop a table
 	 * @param string $tableName the table to drop
@@ -886,13 +990,16 @@ class OC_DB {
 	 * @return bool
 	 */
 	public static function isError($result) {
-		if(!$result) {
+		//MDB2 returns an MDB2_Error object
+		if (class_exists('PEAR') === true && PEAR::isError($result)) {
 			return true;
-		}elseif(self::$backend==self::BACKEND_MDB2 and PEAR::isError($result)) {
-			return true;
-		}else{
-			return false;
 		}
+		//PDO returns false on error (and throws an exception)
+		if (self::$backend===self::BACKEND_PDO and $result === false) {
+			return true;
+		}
+
+		return false;
 	}
 
 	/**
@@ -941,15 +1048,17 @@ class PDOStatementWrapper{
 	/**
 	 * @var PDOStatement
 	 */
-	private $statement=null;
-	private $lastArguments=array();
+	private $statement = null;
+	private $isManipulation = false;
+	private $lastArguments = array();
 
-	public function __construct($statement) {
-		$this->statement=$statement;
+	public function __construct($statement, $isManipulation = false) {
+		$this->statement = $statement;
+		$this->isManipulation = $isManipulation;
 	}
 
 	/**
-	 * make execute return the result instead of a bool
+	 * make execute return the result or updated row count instead of a bool
 	 */
 	public function execute($input=array()) {
 		$this->lastArguments = $input;
@@ -963,16 +1072,19 @@ class PDOStatementWrapper{
 				$input = $this->tryFixSubstringLastArgumentDataForMSSQL($input);
 			}
 
-			$result=$this->statement->execute($input);
+			$result = $this->statement->execute($input);
 		} else {
-			$result=$this->statement->execute();
+			$result = $this->statement->execute();
 		}
-		
-		if ($result) {
-			return $this;
-		} else {
+
+		if ($result === false) {
 			return false;
 		}
+		if ($this->isManipulation) {
+			return $this->statement->rowCount();
+		} else {
+			return $this;
+		}
 	}
 
 	private function tryFixSubstringLastArgumentDataForMSSQL($input) {
@@ -1064,7 +1176,7 @@ class PDOStatementWrapper{
 			die ($entry);
 		}
 	}
-    
+
 	/**
 	 * provide numRows
 	 */
