@@ -74,9 +74,10 @@ class Scanner extends BasicEmitter {
 	 *
 	 * @param string $file
 	 * @param int $reuseExisting
+	 * @param bool $parentExistsInCache
 	 * @return array with metadata of the scanned file
 	 */
-	public function scanFile($file, $reuseExisting = 0) {
+	public function scanFile($file, $reuseExisting = 0, $parentExistsInCache = false) {
 		if (!self::isPartialFile($file)
 			and !Filesystem::isFileBlacklisted($file)
 		) {
@@ -84,7 +85,7 @@ class Scanner extends BasicEmitter {
 			\OC_Hook::emit('\OC\Files\Cache\Scanner', 'scan_file', array('path' => $file, 'storage' => $this->storageId));
 			$data = $this->getData($file);
 			if ($data) {
-				if ($file) {
+				if ($file and !$parentExistsInCache) {
 					$parent = dirname($file);
 					if ($parent === '.' or $parent === '/') {
 						$parent = '';
@@ -110,6 +111,8 @@ class Scanner extends BasicEmitter {
 				if (!empty($newData)) {
 					$this->cache->put($file, $newData);
 				}
+			} else {
+				$this->cache->remove($file);
 			}
 			return $data;
 		}
@@ -157,20 +160,22 @@ class Scanner extends BasicEmitter {
 		$newChildren = array();
 		if ($this->storage->is_dir($path) && ($dh = $this->storage->opendir($path))) {
 			\OC_DB::beginTransaction();
-			while ($file = readdir($dh)) {
-				$child = ($path) ? $path . '/' . $file : $file;
-				if (!Filesystem::isIgnoredDir($file)) {
-					$newChildren[] = $file;
-					$data = $this->scanFile($child, $reuse);
-					if ($data) {
-						if ($data['size'] === -1) {
-							if ($recursive === self::SCAN_RECURSIVE) {
-								$childQueue[] = $child;
-							} else {
-								$size = -1;
+			if(is_resource($dh)) {
+				while (($file = readdir($dh)) !== false) {
+					$child = ($path) ? $path . '/' . $file : $file;
+					if (!Filesystem::isIgnoredDir($file)) {
+						$newChildren[] = $file;
+						$data = $this->scanFile($child, $reuse, true);
+						if ($data) {
+							if ($data['size'] === -1) {
+								if ($recursive === self::SCAN_RECURSIVE) {
+									$childQueue[] = $child;
+								} else {
+									$size = -1;
+								}
+							} else if ($size !== -1) {
+								$size += $data['size'];
 							}
-						} else if ($size !== -1) {
-							$size += $data['size'];
 						}
 					}
 				}
@@ -182,7 +187,7 @@ class Scanner extends BasicEmitter {
 			}
 			\OC_DB::commit();
 			foreach ($childQueue as $child) {
-				$childSize = $this->scanChildren($child, self::SCAN_RECURSIVE);
+				$childSize = $this->scanChildren($child, self::SCAN_RECURSIVE, $reuse);
 				if ($childSize === -1) {
 					$size = -1;
 				} else {
